/* DICElib (DIstributed CAVE Engine library) 
 * Copyright (c) 2001 Bruno Barberi Gnecco <brunobg@lsi.usp.br>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies of the Software, its documentation and marketing & publicity
 * materials, and acknowledgment shall be given in the documentation, materials
 * and software packages that this Software was used.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef DICELIB_H
#define DICELIB_H

#ifdef __cplusplus
extern "C" {
#endif

/*! \file dicelib.h
  \brief This is the main header
  \author Bruno Barberi Gnecco <brunobg@lsi.usp.br>
*/

/** @name Initialization/finalization 
  */
/*@{*/
extern int	DICE_init		( int *argc, char **argv );
extern void 	DICE_close 		( void );
/*@}*/

/** @name DICE attributes
  */
/*@{*/
/**
  \brief Attributes of DICE API.
  The following should be changed \e before calling DICE_init():
  MULTICAST_*, PORT, MAX_CONNECTIONS. Changing them after DICE_init() will
  not result in any changes.
*/
enum dice_attributetype {
	LIBVERSION,		/**< Library version. */
	VERBOSE,		/**< Sets the verbose level.\ Minimum = 0, Maximum = 3.\ Default: 1. */
	PORT,			/**< Port used to connect */
	MAX_CONNECTIONS,	/**< Max concurrent connections (see listen()) */
	OUTPUT,			/**< Error file */
	COLOR,			/**< Output color codes */
	MULTICAST_ADDRESS,	/**< Multicast address */
	MULTICAST_PORT,		/**< Multicast port */
	MULTICAST_TTL		/**< Time to live multicast packet attribute */
};
/**
  \brief Typedef encapsulation.
*/
typedef enum dice_attributetype DICE_AttributeType;

extern int	DICE_set_attribute	( DICE_AttributeType t, void *value );
extern void *	DICE_get_attribute	( DICE_AttributeType t );
extern int	DICE_is_original	( void );
/*@}*/

/**
 @name Type functions
*/
/*@{*/
struct dice_type {
	char			*name;
	char *			(*pack_func) (void *data, int *n);
	void *			(*unpack_func) (char *data, int *n);
	void			(*free_func) (void *data);
};
typedef struct dice_type DICE_Type;
extern DICE_Type DICE_MyTypes[];

#define DICE_no_types		DICE_Type DICE_MyTypes[] = { NULL, NULL, NULL, NULL };
//extern int DICE_type_register 		( DICE_Type *t );
extern int DICE_type_free		( char *type, void *data );
extern char *DICE_type_pack		( char *type, void *data, int *n );
extern void *DICE_type_unpack		( char *type, char *packet, int *n );
/*@}*/


/**
 @name Sync functions
 */
/*@{*/
extern int DICE_sync			( void );

extern int DICE_sync_new		( char *name, char *type, void *value );
extern int DICE_sync_delete		( char *name );
extern int DICE_sync_update		( char *name, void *value );
extern void *DICE_sync_get		( char *name );
extern void *DICE_sync_get_next		( char *name );
/*@}*/

/**
 @name Async functions
 */
/*@{*/
extern int DICE_async_new		( char *name, char *type, void *value );
extern int DICE_async_delete		( char *name );
extern int DICE_async_update		( char *name, void *value );
extern void *DICE_async_get		( char *name );

extern int DICE_async_lock		( char *name );
extern int DICE_async_unlock		( char *name );
/*@}*/

struct dice_func {
	char			*name;
	int			(*func) (void *);
};
typedef struct dice_func DICE_Function;

#define DICE_no_functions	DICE_Function DICE_MyFunctions[] = { NULL, NULL };
extern int DICE_function		( void *data );
extern DICE_Function DICE_MyFunctions[];

#ifdef __cplusplus
} // extern C
#endif

#endif
