/*
    OpenGL Tutor : Rotate

    Copyright (c) 1999 Nate Miller
    
    File        -- rotate.c
    Date        -- 7/01/99
    Author      -- Nate 'm|d' Miller
    Contact     -- vandals1@home.com
    Web         -- http://nate.scuzzy.net

    Many people have asked about how to rotate objects about an axis or point
    or just about their center.  That is why I wrote this demo.  I tried to 
    keep it as simple as possible so that people can see the big picture.  
    It looks horrid I know but its not intended to be flashy, it should be used
    as a tool to learn.  I overcommented this one so beware.  

    If you want to compile this make sure that you link too the following 
    libraries: opengl32.lib glu32.lib glut32.lib

    Usage
    =====
    q           -- quit
    x/y/z       -- changes axis of rotation
    left mouse  -- rotates entire scene
*/

/* 
 * Modified by Bruno Barberi Gnecco to be used with DICElib. 
 * Redistributed under permission.
 */

#define APP_NAME                "OpenGL Tutor : Rotate"
#define X_AXIS                  0
#define Y_AXIS                  1
#define Z_AXIS                  2

#include "dicelib.h"
#include <stdio.h>
#include <GL/glut.h>

#undef HAVE_IEEE754_H
#ifdef HAVE_IEEE754_H
#define FLOAT		"float754"
#else
#define FLOAT		"float"
//#define FLOAT		"double"
#endif

/* vector */
typedef float vec3_t[3]; 

/* enumerations for the mouse buttons */
enum {
    UP = 1, 
    DOWN, 
};

/* window width and height */
int winW = 320; 
int winH = 240;

/* old position of the mouse */
int oldX = -13;
int oldY = -13;

/* mouse state, UP or DOWN */
int mState = UP;

/* current axis of rotation */
int axisRot = X_AXIS;

/* amount to rotate about axis */
float rotate = 0.0f;

/* vector which describes the axis to rotate about */
vec3_t axis = {1.0, 0.0, 0.0};

/* global rotation, for use with the mouse */
vec3_t gRot = {0,0,0};

/* should we animate ? */
int aniOn = 1;

/* packs a 3 float vector */
char *float3_pack	( void *data, int *n ) {
	char			*buf, *t;
	float			*vec;
	int			m;

	vec = (float *)data;
#ifdef HAVE_IEEE754_H
	buf = (char *)malloc(3*4); /* three floats */
#else
	buf = (char *)malloc(3*6); /* three floats */
#endif
	if ( !buf )
		return NULL;
	t = DICE_type_pack(FLOAT, (void *)&vec[0], &m);
	memcpy(buf, t, m);
	free(t);
	t = DICE_type_pack(FLOAT, (void *)&vec[1], &m);
	memcpy(buf+m, t, m);
	free(t);
	t = DICE_type_pack(FLOAT, (void *)&vec[2], &m);
	memcpy(buf+m+m, t, m);
	free(t);
	*n = 3*m;
	return buf;
}
/* unpacks a 3 float vector */
void *float3_unpack	( char *data, int *n ) {
	float			*vec, *f;
	int			m;
	
	vec = (float *)malloc(3*sizeof(float)); /* three floats */
	if ( !vec )
		return NULL;
	f = (float *)DICE_type_unpack(FLOAT, data, &m);
	vec[0] = *f;
	free(f);
	f = (float *)DICE_type_unpack(FLOAT, data+m, &m);
	vec[1] = *f;
	free(f);
	f = (float *)DICE_type_unpack(FLOAT, data+m+m, &m);
	vec[2] = *f;
	free(f);
	*n = 3*m;
	return (void *)vec;
}
/* frees the vector */
void float3_free		( void *data ) {
	if ( data )
		free(data);
}

/* our sync function */
int my_sync ( void ) {
    int i;
    float	*ftmp;

    DICE_sync_update("axisRot", &axisRot);
    DICE_sync_update("rotate", &rotate);
    DICE_sync_update("axis", &axis);
    DICE_sync_update("gRot", &gRot); 

    i = DICE_sync();
    if ( i < 0 )
	return i;

    axisRot = *((int *)DICE_sync_get("axisRot"));
    rotate = *((float *)DICE_sync_get("rotate"));
    ftmp = (float *)DICE_sync_get("axis");
    memcpy(axis, ftmp, 3*sizeof(float));
    ftmp = (float *)DICE_sync_get("gRot"); 
    memcpy(gRot, ftmp, 3*sizeof(float)); 
}

/* must declare */
DICE_no_functions;
DICE_Type DICE_MyTypes[] = {
	{ "float3", float3_pack, float3_unpack, float3_free },
	{ NULL, NULL, NULL, NULL }
};

/*
=============
drawSphere
=============

Draws our spheres to the screen.  The first sphere rotates about the current 
axis.  The second rotates about its center.  
*/
void drawSphere (void)
{
    /* 
        This sphere is going to be rotated around the *current axis*.  This is 
        because the model was first rotated and then translated.
    */
    glPushMatrix ();

    glRotatef (rotate, axis[0], axis[1], axis[2]);

    glTranslatef (-15.0, 10.0, 5.0);

    glColor3f (1.0, 0.0, 0.0);

    glutWireSphere (5.0, 6.0, 6.0);

    glPopMatrix ();

    /* 
        This sphere is going to be rotated around the its own center.  This is 
        because the model was first translated and then rotated.  
    */
    glPushMatrix ();

    glTranslatef (5.0, 10.0, 0.0);

    glRotatef (rotate, axis[0], axis[1], axis[2]);

    glColor3f (0.0, 1.0, 0.0);

    glutWireSphere (2.0, 6.0, 6.0);

    glPopMatrix ();
}
/*
=============
drawAxis
=============

Draws out the X / Y / Z axis just for reference.  
*/
void drawAxis (void)
{
    glColor3f (0.5, 0.5, 0.5);
    glBegin (GL_LINES);
        glColor3f (0.5, 0.0, 0.0);
        glVertex3f (-20.0, 0.0, 0.0);
        glVertex3f (20.0, 0.0, 0.0);

        glColor3f (0.0, 0.5, 0.0);
        glVertex3f (0.0, 20.0, 0.0);
        glVertex3f (0.0, -20.0, 0.0);

        glColor3f (0.0, 0.0, 0.5);
        glVertex3f (0.0, 0.0, -20.0);
        glVertex3f (0.0, 0.0, 20.0);
    glEnd ();
}
/*
=============
glutDisplay
=============

The display function. 
*/
void glutDisplay (void)
{
    /* send updates, synchronize and update variables */
    my_sync();
 
    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();

    /* move our eye position 50 units away */
    glTranslatef (0, 0, -50);	  
    /* apply the rotation from the mouse */
    glRotatef (gRot[0], 1.0, 0.0, 0.0);
    glRotatef (gRot[1], 0.0, 1.0, 0.0);

    drawAxis ();
    drawSphere ();

    glutSwapBuffers();
}
/* 
=============
glutIdle
=============

Idle function, makes it animate.  
*/
void glutIdle (void)
{	
    if (aniOn)
    {
        rotate += 1.0f;

        if (rotate > 360)
            rotate = 0.0f;
        glutPostRedisplay ();
    }
    glutPostRedisplay ();
}
/* 
=============
glutResize
=============

Resize function, called on app start and on resize.
*/
void glutResize (int w, int h)
{	
    winW = w;
    winH = h;

    glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glViewport (0, 0, winW, winH);

    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();

    gluPerspective (90, winW / winH, 1, 9999);

    glutPostRedisplay ();
}
/* 
=============
glutKeyboard
=============

Keyboard handler.
*/
void glutKeyboard (unsigned char key, int x, int y)
{
    switch (key)
    {
        case 'q':
        case 'Q':
	    DICE_close();
            exit (1);
        break;
        case 'x':
        case 'X':
            /* axis of rotation is now X */
            axisRot = X_AXIS;
            axis[0] = 1.0;
            axis[1] = axis[2] = 0.0;
        break;
        case 'y':
        case 'Y':
            /* axis of rotation is now Y */
            axisRot = Y_AXIS;
            axis[1] = 1.0;
            axis[0] = axis[2] = 0.0;
        break;
        case 'z':
        case 'Z':
            /* axis of rotation is now Z */
            axisRot = Z_AXIS;
            axis[2] = 1.0;
            axis[1] = axis[0] = 0.0;
        break;
	case 'd':
	    fprintf(stderr, "%f [%f %f %f]\n", rotate, gRot[0], gRot[1], gRot[2]);
	    break;
        case ' ':
            /* animation on/off */
            aniOn = !aniOn;
        break;
    }
    glutPostRedisplay ();
}
/*
=============
clamp
=============

Clamps the rotation to -360 to 360
*/
void clamp (vec3_t v)
{
    int i;

    for (i = 0; i < 3; i ++)
        if (v[i] > 360 || v[i] < -360)
            v[i] = 0.0f;
}
/*
=============
glutMotion
=============

Called when the mouse moves in our app area. 
*/
void glutMotion(int x, int y) 
{
    if (mState == DOWN) 
    {
        gRot[0] -= ((oldY - y) * 180.0f) / 100.0f;
        gRot[1] -= ((oldX - x) * 180.0f) / 100.0f;
        clamp (gRot);
        glutPostRedisplay ();
    } 
    oldX = x; 
    oldY = y;
}
/*
=============
glutMouse
=============

Called when the mouse is clicked.
*/
void glutMouse(int button, int state, int x, int y) 
{
    if(state == GLUT_DOWN) 
    {
        switch(button) 
        {
            case GLUT_LEFT_BUTTON:
            case GLUT_RIGHT_BUTTON:
                mState = DOWN;
                oldX = x;
                oldY = y;
            break;
        }
    } else if (state == GLUT_UP) 
        mState = UP;
}
/* 
=============
glInit
=============

Sets up various OpenGL stuff.
*/
void glInit (void)
{
    glEnable (GL_DEPTH_TEST);
}
int main (int argc, char **argv)
{
    /* init DICElib */
    DICE_init(&argc, argv);

    /* Create variables. */

    if (DICE_is_original()) {
    DICE_sync_new("axisRot", "signed_long", &axisRot);
    DICE_sync_new("rotate", FLOAT, &rotate);
    DICE_sync_new("axis", "float3", axis);
    DICE_sync_new("gRot", "float3", gRot);
}
    /* synchronize everybody, making the new variables valid */
    DICE_sync();

    glutInitDisplayMode (GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
    glutInitWindowSize (winW,winH);
    if ( DICE_is_original() ) {
	    glutCreateWindow ("original");
	    glutKeyboardFunc (glutKeyboard);
	    glutMotionFunc (glutMotion);
	    glutMouseFunc (glutMouse);
    }    
    else {
    	    glutCreateWindow ("client");
    }
    glutIdleFunc (glutIdle);
    glutReshapeFunc (glutResize);
    glutDisplayFunc (glutDisplay);
    glInit ();
    glutMainLoop();
}
